/*------------------------------------------------------------------
cscufl.h
Copyright 2012 Leonardo Lenoci <leonardo.lenoci@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------- */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MAX_CSCUFL_BUFFER 190000
#define MAX_CSCUFL_DESC 150

/* USEFUL TAGS */
#define XML_TAG "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
#define SCUFL_O "<s:scufl xmlns:s=\"http://org.embl.ebi.escience/xscufl/0.1alpha\" version=\"0.2\" log=\"0\">\n"
#define SCUFL_C "</s:scufl>\n"

#define cscufl_wf_desc(desc,aut,tit) sprintf(desc,"<s:workflowdescription  author=\"%s\" title=\"%s\">%s</s:workflowdescription>\n",\
						 aut,tit, tit)
#define cscufl_write(cscuflbuffer,...) cscuflbuffer->res+=sprintf((cscuflbuffer->buffer)+(cscuflbuffer->res),__VA_ARGS__)

#define cscufl_input_port(cscuflbuffer,port) cscufl_write(cscuflbuffer,"<s:source name=\"%s\"></s:source>\n", port)

#define cscufl_output_port(cscuflbuffer,port) cscufl_write(cscuflbuffer,"<s:sink name=\"%s\"></s:sink>\n", port)

#define cscufl_bean_shell(cscuflbuffer,name,script) cscufl_write(cscuflbuffer,"<s:processor name=\"%s\"><s:beanshell><s:scriptvalue>%s</s:scriptvalue></s:beanshell></s:processor>\n", name, script)

#define cscufl_local_srv(cscuflbuffer,name,srv) cscufl_write(cscuflbuffer, "<s:processor name=\"%s\">\n<s:local>org.embl.ebi.escience.scuflworkers.java.%s</s:local>\n</s:processor>\n",name,srv);


#define cscufl_rshell_init(cscuflbuffer,name,hostname,\
			   port,user,passwd,session,script) cscufl_write(cscuflbuffer,"<s:processor name=\"%s\">\n\
<s:rshell s:hostname=\"%s\" s:port=\"%d\" s:username=\"%s\" s:password=\"%s\" s:keepSessionAlive=\"%s\">\n%s\n",\
									 name,hostname,port,user,passwd,session,script)

#define cscufl_WSDL_kegg(cscuflbuffer,name,operation) cscufl_write(cscuflbuffer,"<s:processor name=\"%s\">\n\
<s:arbitrarywsdl>\n\
<s:wsdl>http://soap.genome.jp/KEGG.wsdl</s:wsdl>\n\
<s:operation>%s</s:operation>\n</s:arbitrarywsdl>\n</s:processor>",name,operation);


#define cscufl_link(cscuflbuffer,source,sink) cscufl_write(cscuflbuffer,"<s:link source=\"%s\" sink=\"%s\" />\n", source,sink)

#define cscufl_err(message) fprintf(stderr,"err-->libcscufl: %s\n",message) 


/* to be added: list of all services! http://build.mygrid.org.uk/taverna/api/org/embl/ebi/escience/scuflworkers/ */


char cscufl_desc[MAX_CSCUFL_DESC];

typedef struct
{
  size_t res; 
  char buffer[MAX_CSCUFL_BUFFER];
} cscufl_buffer;


typedef FILE cscufl_FILE;


typedef struct
{
  const char *syn_t;
  const char *sym_t;
  const char *name;
} rshell_port_t;

typedef enum {true=1,false=0} cscufl_bool;


/* PROTOTYPES */
cscufl_buffer*
cscufl_calloc(void);

cscufl_FILE*
cscufl_fopen(char *filename);

void
cscufl_fclose(cscufl_FILE* pfile);

void
cscufl_free(cscufl_buffer* buff);

int 
cscufl_check_buffer(cscufl_buffer* buff);

void
cscufl_rshell_inout(cscufl_buffer* buff, int num_ports, rshell_port_t* port, cscufl_bool inout);
