/*------------------------------------------------------------------
test.c
Copyright 2012 Leonardo Lenoci <leonardo.lenoci@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------- */

#include "cscufl.h"

int 
main(){
  
  cscufl_buffer* my_buff;
  
  
  const  char* author="leonardo";
  const  char* title="test_wf";

  typedef enum {hello=0, msconvert=1} example_t;

  example_t example;

  example=strstr(getenv("EXAMPLE") ,"hello") != NULL ?  hello :  msconvert;


  rshell_port_t in_ports[]=
    {
      {"l(\'text/plain\')","STRING_LIST","dirPath1"},
      {"l(\'text/plain\')","STRING_LIST","dirPath2"}
    };

  rshell_port_t out_ports[]=
    {
      {"\'image/PNG\'","PNG_FILE","boxNormPlot"}
    };


  
  const char *hello_script="\n\
result = \"Hello \" + name;\n\
";

  
  const char *msconv_script="\n\
public static String arrayToString2(String[] a, String separator) {\n\
    StringBuffer result = new StringBuffer();\n\
    if (a.length &gt; 0) {\n\
        result.append(a[0]);\n\
        for (int i=1; i&lt;a.length; i++) {\n\
            result.append(separator);\n\
            result.append(a[i]);\n\
        }\n\
    }\n\
    return result.toString();\n\
}\n\
command=\"wine\";\n\
List args = new ArrayList();\n\
args.add(\"msconvert\");\n\
args.add(raw);\n\
args.add(\"--32\");\n\
args.add(\"-z\");\n\
args.add(\"--filter\");\n\
args.add(\"\\\"peakPicking\");\n\
args.add(\"prefer_vendor\");\n\
args.add(\"true\");\n\
args.add(\"1-\\\"\");\n\
args.add(\"--filter\");\n\
args.add(\"\\\"zeroSamples\");\n\
args.add(\"removeExtra\");\n\
args.add(\"1-\\\"\");\n\
args.add(\"--outfile\");\n\
String mzML_file = raw.replace(\"yep\",\"mzML\");\n\
args.add(mzML_file);\n\
Process proc = null;\n\
Runtime rt = Runtime.getRuntime();\n\
String[] cmdArray = null;\n\
cmdArray = new String[] { command };\n\
if ((args == void) || (args == null)) {\n\
        args = new ArrayList();\n\
}\n\
int argSize = cmdArray.length + args.size();\n\
ArrayList appArray = new ArrayList(argSize);\n\
for (int i = 0; i &lt; cmdArray.length; i++) {\n\
        appArray.add(cmdArray[i]);\n\
}\n\
for (int i = 0; i &lt; args.size(); i++) {\n\
        appArray.add(args.get(i));\n\
}\n\
String[] applist = new String[argSize];\n\
appArray.toArray(applist);\n\
cmd = arrayToString2(applist,\" \");\n\
proc = rt.exec(applist);\n\
InputStream in = proc.getInputStream();\n\
int c;\n\
StringBuffer sb = new StringBuffer();\n\
while ((c = in.read()) != -1) {\n\
        sb.append((char) c);\n\
}\n\
in.close();\n\
out = sb.toString();\n\
";



  my_buff=cscufl_calloc();

  /* workflow header */
  cscufl_write(my_buff,XML_TAG);
  cscufl_write(my_buff,SCUFL_O);
  cscufl_wf_desc(cscufl_desc,author,title);
  cscufl_write(my_buff, "%s",cscufl_desc);
  
  switch(example){
  case hello:
    /* ------HELLO WORLD WORKFLOW------ */
    /* input ports */
    cscufl_input_port(my_buff,"NAME");
    /* bean shell */
    cscufl_bean_shell(my_buff,"HELLO",hello_script);
    if (cscufl_check_buffer(my_buff)) exit(EXIT_FAILURE);
    /* output ports */
    cscufl_output_port(my_buff,"GREETING");
    /* links */
    cscufl_link(my_buff,"NAME","HELLO:name");
    cscufl_link(my_buff,"HELLO:result", "GREETING");
    /* rshell */
    cscufl_rshell_init(my_buff,"pieChart","localhost",\
		       6311,"","","false","do somenthing");
    cscufl_rshell_inout(my_buff,2,&in_ports[0],true);
    cscufl_rshell_inout(my_buff,1,&out_ports[0],false);
    /* WSDL */
    cscufl_WSDL_kegg(my_buff,"kegg_thing","get_pathways_by_genes");
    /* local service */
    cscufl_local_srv(my_buff,"MERGE","StringListMerge");
    /* ------END HELLO WORLD WORKFLOW ------*/
    break;
  case msconvert:
    /* ------MS CONVERT WORKFLOW ------*/
    cscufl_input_port(my_buff,"RAW");
    cscufl_bean_shell(my_buff,"MSCONVERT",msconv_script);
    if (cscufl_check_buffer(my_buff)) exit(EXIT_FAILURE);
    cscufl_output_port(my_buff,"MSCONVERT_LOG");
    cscufl_output_port(my_buff,"CMD");
    cscufl_link(my_buff,"RAW","MSCONVERT:raw");
    cscufl_link(my_buff,"MSCONVERT:out", "MSCONVERT_LOG");
    cscufl_link(my_buff,"MSCONVERT:cmd", "CMD");
    /* ------END MS CONVERT WORKFLOW ------*/
    break;
  default:
    cscufl_err("Gotta specify the env var EXAMPLE!\n");exit(EXIT_FAILURE);
  }
  

  /* workflow footer */
  cscufl_write(my_buff,SCUFL_C);
  

  /* write workflow on stdout */
  puts(my_buff->buffer);

  /* clean up */
  cscufl_free(my_buff);
  
  
  
  

  return(EXIT_SUCCESS);
}
